/* 

Functions for creating backups and restore on legacy (UPB) devices.
===================================================================
 
Required API version: v1.2.0 

Globals: deviceIP, factoryDefault

*/

var getPresetCount = function() {
    var count = 0;
    
    try {
        var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);
        
        var presetCountGetProperty = new GetProperty(lw3Connection, new Property("Count", new Path("/PRESETS/DEVICE")));
        presetCountGetProperty.execute();

        count = presetCountGetProperty.getProperty().getCachedValue();
        
    } catch(exception) {
        logger.debug("There is no device preset.");
    }
    
    return count;
}

var getPresetProperties = function(presetId) {        
    var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);
    
    var getPresetNameProperty = new GetProperty(lw3Connection, new Property("name", new Path("/PRESETS/DEVICE/" + presetId)));
    getPresetNameProperty.execute();
    var presetNameProperty = getPresetNameProperty.getProperty().getCachedValue();

    var presetFileLengthGetProperty = new GetProperty(lw3Connection, new Property("fileLen", new Path("/PRESETS/DEVICE/" + presetId)));
    presetFileLengthGetProperty.execute();
    var fileLength = presetFileLengthGetProperty.getProperty().getCachedValue();
    
    var presetErrorCountGetProperty = new GetProperty(lw3Connection, new Property("errorNum", new Path("/PRESETS/DEVICE/" + presetId)));
    presetErrorCountGetProperty.execute();
    var errorCount = presetErrorCountGetProperty.getProperty().getCachedValue();       

    var presetWriteProtectedGetProperty = new GetProperty(lw3Connection, new Property("writeProtected", new Path("/PRESETS/DEVICE/" + presetId)));
    presetWriteProtectedGetProperty.execute();
    var writeProtected = presetWriteProtectedGetProperty.getProperty().getCachedValue();           
    
    return {
        'name': presetNameProperty,
        'empty': (0 == fileLength),
        'convertable': ((0 < fileLength) && (0 == errorCount)),
        'writeProtected': ((0 < fileLength) && writeProtected)
    };
}

var isIndexFileValid = function() {
    var indexFileValid = false;

    try {
        var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);
        
        var indexFileLengthGetProperty = new GetProperty(lw3Connection, new Property("ActualFileLen", new Path("/MANAGEMENT/INDEXFILE")));
        indexFileLengthGetProperty.execute();

        var length = indexFileLengthGetProperty.getProperty().getCachedValue();
        indexFileValid = (0 < length);
        
    } catch (exception) {
        logger.debug("Failed to get index file length.");
    }

    return indexFileValid;
}

var deletePreset = function(presetId, socketTimeout /* ms */) {
    var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, socketTimeout, 3);
    if(socketTimeout < 1000) {
        throw "deletePreset() function is called with invalid argument: socketTimeout.";
    }
    
    var deletePresetMethod = new Method("delete", new Path("/PRESETS/DEVICE"));        
    var callDeletePreset = new CallMethod(lw3Connection, deletePresetMethod, presetId + ";IDDQD");
    callDeletePreset.execute();    
    
    var presetFileLengthGetProperty = new GetProperty(lw3Connection, new Property("fileLen", new Path("/PRESETS/DEVICE/" + presetId)));
    presetFileLengthGetProperty.execute();
    var fileLength = presetFileLengthGetProperty.getProperty().getCachedValue();
    
    if(0 != fileLength) {
        throw "Failed to delete preset " + presetId;
    }
}

var createBackup = function(converterScriptName, skipDevicePresets, httpSocketTimeout /* ms */, presetEraseTimeout /* ms */) {
    if(httpSocketTimeout < 1000) {
        throw "createBackup() function is called with invalid argument: httpSocketTimeout.";
    }
    if(presetEraseTimeout < 1000) {
        throw "createBackup() function is called with invalid argument: presetEraseTimeout.";
    }
    
    logger.info("Creating backup files in folder: {}", backupFolder);

    rootCheckpoint.indeterminate();

    var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);

    var partNumberGetProperty = new GetProperty(lw3Connection, new Property("PartNumber", new Path("/")));
    partNumberGetProperty.execute();
    var partNumber = partNumberGetProperty.getProperty().getCachedValue();

    var firmwareVersionGetProperty = new GetProperty(lw3Connection, new Property("FirmwareVersion", new Path("/SYS/MB")));
    firmwareVersionGetProperty.execute();
    var fwVersionWithRevision = firmwareVersionGetProperty.getProperty().getCachedValue().split(' ');
    if(fwVersionWithRevision.length == 1) {
        fwVersion = fwVersionWithRevision;
        fwRevision = 0;
    }
    else {
        fwVersion = fwVersionWithRevision[0];
        fwRevision = fwVersionWithRevision[1].substr(1);
    }

    var backupCreateFlowArgumentBuilder = BackupCreateFlowArgument.newBuilder()
        .setTcpConnection(lw3Connection)
        .setBackupFolder(backupFolder)
        .addKeyValue("PARTNUMBER", partNumber)
        .addKeyValue("VERSION", fwVersion)
        .addKeyValue("REVISION", fwRevision);

    if(factoryDefault == false && restoreFromBackup == true && converterScriptName != "") {
        backupCreateFlowArgumentBuilder.setConvertScriptInputStreamProvider(packageFile.getFileHandler(converterScriptName))
    }

    var backupCreateFlowArgument = backupCreateFlowArgumentBuilder.build();

    var backupCreateFlow = new BackupCreateFlow(backupCreateFlowArgument);
    backupCreateFlow.execute();
    factoryDefault = factoryDefault || backupCreateFlow.isFactoryDefaultRequested();

    var presetFileMetaDataList = [];
    var erasePresetList = [];       
    
    for(presetId = 1; presetId <= getPresetCount(); presetId++) {
        var presetProperties = getPresetProperties(presetId);
        logger.debug("Preset {} properties:\n{}", presetId, JSON.stringify(presetProperties));
        
        if(false == presetProperties['empty']) {            
            var presetFileMetaDataBuilder = PresetFileMetaData.newBuilder()
                .setUrl("/preset" + presetId + ".lw3")
                .setName(presetProperties['name']+" ("+"preset"+presetId + ")");                

            if(true == presetProperties['convertable']) {
                if(!factoryDefault && !skipDevicePresets){
                    presetFileMetaDataBuilder.setConvertScriptInputStreamProvider(packageFile.getFileHandler(converterScriptName))
                }
            }
            else {
                /* Base64 encode content */
                presetFileMetaDataBuilder.setBinaryContent(true);
            }            
            
            var presetFileMetaData = presetFileMetaDataBuilder.build();
            presetFileMetaDataList.push(presetFileMetaData);
            
            logger.info("Backup of device preset {} is queued.", presetFileMetaData.getName());
        }
        
        if(true == presetProperties['writeProtected']) {
            erasePresetList.push(presetId);
        }
    }    
    
    var backupPresetFlowArgument = BackupPresetFlowArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setBackupFolderPath(backupFolder)
        .setPresetFileMetaDataList(presetFileMetaDataList)
        .addKeyValue("PARTNUMBER", partNumber)
        .addKeyValue("VERSION", fwVersion)
        .addKeyValue("REVISION", fwRevision)
        .setSocketTimeout(httpSocketTimeout)
        .build();

    var backupPresetFlow = new BackupPresetFlow(backupPresetFlowArgument);
    backupPresetFlow.execute();

    var indexFileMetaDataList = [];
    if(isIndexFileValid()) {
        var indexFileMetaData = PresetFileMetaData.newBuilder()
            .setUrl("/index.html")
            .setName("index.html")
            .setBinaryContent(true)
            .build();
        indexFileMetaDataList.push(indexFileMetaData);
        logger.info("Backup of Index file is queued.");
    } else {
        logger.debug("No backup from Index file.");
    }    

    var backupIndexFileFlowArgument = BackupPresetFlowArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setBackupFolderPath(backupFolder)
        .setPresetFileMetaDataList(indexFileMetaDataList)
        .addKeyValue("PARTNUMBER", partNumber)
        .addKeyValue("VERSION", fwVersion)
        .addKeyValue("REVISION", fwRevision)
        .setSocketTimeout(httpSocketTimeout)
        .build();

    var backupIndexFileFlow = new BackupPresetFlow(backupIndexFileFlowArgument);
    backupIndexFileFlow.execute();    

    /* Override factoryDefault */
    factoryDefault = factoryDefault || backupPresetFlow.isFactoryDefaultRequested();
    
    /* Erase write protected presets */
    if(!factoryDefault) {
        erasePresetList.forEach(
            function(presetId) {
                logger.debug("Deleting preset {}...", presetId);
                deletePreset(presetId, presetEraseTimeout);
            });
    }

    return {
        'backupFilePath': backupCreateFlow.getBackupFilePath(),
        'presetFileMetaDataList': backupPresetFlow.getPresetFileMetaDataList(),
        'indexFileMetaDataList': backupIndexFileFlow.getPresetFileMetaDataList()
    };
}

var restoreSettings = function(backupFilePath, socketTimeout /* ms */) {
    if(backupFilePath == '') {
        throw "restoreSettings() function is called with invalid argument: backupFilePath.";
    }  
    if(socketTimeout < 1000) {
        throw "restoreSettings() function is called with invalid argument: socketTimeout.";
    }    
    
    var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, socketTimeout, 5);
    
    var restoreNetworkSettingsArgument = RestoreNetworkSettingsArgument.newBuilder()
        .setRestoreNetworkMode(RestoreNetworkSettingsArgument.RestoreNetworkMode.BACKUP)
        .build();

    var restoreOverLw3FlowArgument = RestoreOverLw3FlowArgument.newBuilder()
        .setTcpConnection(lw3Connection)
        .setBackupFilePath(backupFilePath)
        .setRestoreNetworkSettingsArgument(restoreNetworkSettingsArgument)
        .build();

    var restoreOverLw3Flow = new RestoreOverLw3Flow(restoreOverLw3FlowArgument);
    restoreOverLw3Flow.execute();            
}

var restoreFiles = function(fileMetaDataList, httpSocketTimeout /* ms */, checkpoint) {
    if(fileMetaDataList instanceof Array) {
        throw "restorePresets() function is called with invalid argument: fileMetaDataList.";
    }
    if(httpSocketTimeout < 1000) {
        throw "restorePresets() function is called with invalid argument: httpSocketTimeout.";
    }
    if(!checkpoint instanceof Checkpoint) {
        throw "restorePresets() function is called with invalid argument: checkpoint.";
    }
    
    fileMetaDataList.forEach(
        function(fileMetaData) {
            var restoreArgument = RestorePresetFlowArgument.newBuilder()
                .setDeviceIp(deviceIp)
                .setPresetFilePath(fileMetaData.getFilePath())
                .setRestoreUrl(fileMetaData.getUrl())
                .setSocketTimeout(httpSocketTimeout)
                .setRetries(3)
                .setPresetName(fileMetaData.getName());

            logger.info("Restoring {}...", fileMetaData.getName());

            var restoreFlow = new RestorePresetFlow(restoreArgument.build());
            restoreFlow.execute();
            rootCheckpoint.waitMillisLong(3000);

            checkpoint.setProgress(checkpoint.getProgress() + (1.0 / fileMetaDataList.length));
        });
}
