/* 

Functions for UID handling of legacy (UPB) devices.
===================================================
 
Required API version: v1.2.0 

Globals: deviceIP, packageFile

Provided global variables: productName, productPartNumber, productSerialNumber

Example of usage: 
 
//// create UID EEPROM/SDA GpioConfig
var SDA_UID = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_11)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_OPENDRAIN)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

//// create UID EEPROM/SCL GpioConfig
var SCL_UID = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_10)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_OPENDRAIN)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

//// define UID Eeprom i2c address
var UID_EEPROM_I2C_ADDRESS = 0xA0;

//// create UID Eeprom I2C Eeprom config
var uidFvsEepromConfig = I2cEepromConfig.newBuilder()
    .setScl(SCL_UID)
    .setSda(SDA_UID)
    .setI2cAddress(UID_EEPROM_I2C_ADDRESS)
    .setEepromType(I2cEepromConfig.EepromType.EEPROM_24LC512K)
    .build();

//// get UID from the device
getUID(uidFvsEepromConfig);

if(91310035 == productPartNumber) {
    
}
    
*/

var getUID = function(uidFvsEepromConfig) {    
    if(!uidFvsEepromConfig instanceof I2cEepromConfig) {
        throw "getUID() function is called with invalid argument: uidFvsEepromConfig.";
    }

    logger.info("Gathering UID information from device...");

    var upbUidArgument = UpbUidArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setConfig(uidFvsEepromConfig)
        .build();

    var upbGetUidFlow = new UpbGetUidFlow(upbUidArgument);

    /* getting UID information */
    upbGetUidFlow.execute();
    var uid = upbGetUidFlow.getUid();

    /* Printing UID information */
    logger.info("Product name: \t{}\nProduct part number: \t{}\nSerial number: \t{}\nHardware version: \t{}",
        uid.getProductName(),
        uid.getProductPartNumber(),
        uid.getProductSerialNumber(),
        uid.getHwVersion().getHwVersionAsString());
        
    return uid;
}

var hardwareVersionStrToInt = function(versionStr /* "V11_CAX0" */) {
    if(versionStr.length != 8) {
        throw "hardwareVersionStrToInt() function is called with invalid argument: versionStr.";
    }
    
    var result = 0;
    
    for(var i = 0; i < versionStr.length; i++) {
        result *= 255;
        result += versionStr.charCodeAt(i) & 255;
    }
    
    return result;
}

var hardwareVersionToInt = function(version) {
    if(!version instanceof Uid.HwVersion) {
        throw "hardwareVersionToInt() function is called with invalid argument: version.";
    }    
    
    return hardwareVersionStrToInt(version.getHwVersionAsString());
}

var isCurrentFirmwareNewer = function(fwFile) {
    if(!fwFile instanceof InternalFlashFile) {
        throw "isFirmwareDowngrade() function is called with invalid argument: fwFile.";
    }
    
    var uidArgument = UpbUidArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setConfig(fwFile.getFvsEepromConfig())
        .build();
        
    var getVersionStorage = new UpbGetVersionStorageFlow(uidArgument);
    getVersionStorage.execute();
    
    var fwVersionOnDevice = getVersionStorage.getFirmwareVersionStorage().toFirmwareVersionArgument(fwFile.getFvsBlockNumber());
    
    var fwVersionInPackage = packageFile.getFirmwareVersionArgument(fwFile.getFileNameInPackage());
    
    return firmwareVersionToInt(fwVersionInPackage) < firmwareVersionToInt(fwVersionOnDevice);
}
